package hello.jsp;

import java.util.*;
import java.io.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import java.lang.reflect.Constructor;

/**
 * This class returns an implementation of an action command object 
 * by name. Action commands are delegates assigned by the controller
 * servlet to process specific commands issued by the presentation layer.
 * It uses the SAX API to parse the XML file which contains the action
 * mappings.
 * $Revision: 1.2 $ $Date: 2002/09/28 20:27:30 $ (helder@acm.org).
 */
public class CommandFactory extends org.xml.sax.helpers.DefaultHandler {
    
    /**
     * File where the named commands are bound to the action objects.
     * The name of this file is specified in the deployment descriptor.
     */
    public String mappingsFile;
    private boolean inCommand = false;
    
    /**
     * Collection of command action objects.
     */
    private HashMap commands = new HashMap();
    
    /////////////// Singleton implementation /////////////////////////////////////
    /**
     * Singleton implementation.
     */
    private static CommandFactory instance = null;
    private CommandFactory(String mappingsFile) throws IOException {
        try {
            File file = new File(mappingsFile);
            if (!file.exists()) {
                throw new FileNotFoundException("Unable to find " + mappingsFile + ".");
            }
            FileInputStream stream = new FileInputStream(file);
            SAXParserFactory saxFactory = SAXParserFactory.newInstance();
            SAXParser parser = saxFactory.newSAXParser();
            parser.parse(stream, this);
        } catch (Exception e) {
            if (e instanceof RuntimeException) {
                throw (RuntimeException) e;
            } else {
                throw new IOException("CommandFactory - Unable to initialize command table. " + e);
            }
        }
    }
    
    /**
     * Returns an instance of this factory.
     */
    public static CommandFactory getInstance(String mappingsFile) throws IOException {
        if (instance == null) {
            return new CommandFactory(mappingsFile);
        }
        return instance;
    }
    
    /////////////// End Singleton implementation //////////////

    
    /////////////// Main instance utility method ///////////////
    /**
     * Returns a command action object mapped to a supplied command name.
     */
    public Command getCommand(String name) throws CommandNotFoundException {
        Command cmd = (Command)commands.get(name);
        if (cmd == null) {
            throw new CommandNotFoundException("No action object is bound to the command '" + name + "'");
        }
        return cmd;
    }
    /////////////////////////////////////////////////////
    
    ////////////////// SAX XML reader artifacts /////////////////////////
    /**
     * Instance of internal class used by the SAX parser.
     */
    private CommandData currentCommandData;
    private String currentText;
    
    /**
     * This internal class is used as a temporary repository for the data read via
     * the SAX parser.
     */
    class CommandData {
        private String commandName;
        private String className;
        private String successParam;
        private String failureParam;
        
        public CommandData() {}
        
        public void setCommandName(String commandName) {
            this.commandName = commandName;
        }
        public void setClassName(String className) {
            this.className = className;
        }
        public void setSuccessParam(String successParam) {
            this.successParam = successParam;
        }
        public void setFailureParam(String failureParam) {
            this.failureParam = failureParam;
        }
        public String getCommandName() {
            return commandName;
        }
        public String getClassName() {
            return className;
        }
        public String getSuccessParam() {
            return successParam;
        }
        public String getFailureParam() {
            return failureParam;
        }
    }
        
    ////////////// SAX Handler Implementation /////////////////
    
    /**
     * Called when a start tag is found.
     */
    public void startElement(String ns, String ln, String name, Attributes atts) {
        if (name.equals("command")) {
            inCommand = true;
            currentCommandData = new CommandData();
        }
    }
    
    /**
     * Called when an end tag is found.
     */
    public void endElement(String ns, String ln, String name) {
        if (name.equals("command")) {
            inCommand = false;
            try {
                Class commandClass = Class.forName(currentCommandData.getClassName());
                Constructor constructor = commandClass.getConstructor(new Class[] {String.class, String.class});
                String[] args = { currentCommandData.getSuccessParam(), currentCommandData.getFailureParam() };
                Command command = (Command)constructor.newInstance(args);
                
                commands.put( currentCommandData.getCommandName(), command);
            } catch (Exception e) {
                if (e instanceof RuntimeException) {
                    throw (RuntimeException) e;
                }
                System.out.println("Reflection Error: " + e);
            }
            currentCommandData = null;
        }
        if (inCommand) {
            if (name.equals("name")) {
                currentCommandData.setCommandName(currentText);
            } else if (name.equals("class")) {
                currentCommandData.setClassName(currentText);
            } else if (name.equals("success-url")) {
                currentCommandData.setSuccessParam(currentText);
            } else if (name.equals("failure-url")) {
                currentCommandData.setFailureParam(currentText);
            }
            currentText = null;
        }
    }
    
    /**
     * Called when a character node is found.
     */
    public void characters(char[] chars, int start, int length) {
        if (inCommand) {
            currentText = (new String(chars, start, length)).trim();
        }
    }
    
    /////////////////// End SAX /////////////////////////////////
}
