package hello.jsp;

import java.io.*;
import java.util.ArrayList;

/**
 * This Data Access Object implements persistence with a JAXB (early access
 * edition) XML binding mechanism. We assume the bean passed as a parameter
 * (MessageBean) implements the JAXB interfaces (has validate(), marshal()
 * and unmarshal() methods.)
 * $Revision: 1.2 $ $Date: 2002/10/06 22:52:41 $ (helder@acm.org).
 */ 
public class XMLMessageBeanDAO implements MessageBeanDAO {
    
    /**
     * The directory where XML messages will be stored.
     */
    private String messageDir;
    
    /**
     * @param messageDir The directory where XML messages will be stored.
     */
    public XMLMessageBeanDAO(String messageDir) {
        this.messageDir = messageDir;
    }
    
    /**
     * Returns data storage location.
     */
    public Object getLocator() {
        return messageDir;
    }
    
    /**
     * Saves message bean data in a XML file.
     */
    public void persist(MessageBean bean) throws PersistenceException {
        FileOutputStream outFile = null;
        File messageDirectory = new File(messageDir);
        if (!messageDirectory.exists()) {
            messageDirectory.mkdir();
        }
        try {
            String idx = this.getNextIndex();
            outFile = new FileOutputStream(messageDir + "/" + "message_"+ idx + ".xml");
            bean.validate();
            bean.marshal(outFile);
        } catch (javax.xml.bind.UnmarshalException e) {
            throw new PersistenceException(e);
        } catch (IOException e) {
            throw new PersistenceException(e);   
        } finally {
            try {
                outFile.close();
            } catch (IOException e) {
                throw new PersistenceException(e);   
            }
        }
    }
    
    /**
     * Retrieves all messages stored in directory.
     */
    public MessageBean[] retrieveAll() throws PersistenceException {
        ArrayList beans = new ArrayList();
        int[] idx = this.getIndexes();
        for (int i = 0; i < idx.length; i++) {
            MessageBean bean = this.retrieve(idx[i]);
            beans.add(bean);
        }
        return (MessageBean[])beans.toArray(new MessageBean[beans.size()]);
    }
    
    /**
     * Retrieves messageBean from a XML file given an identification key (which
     * is the index of the file.)
     */
    public MessageBean retrieve(int key) throws PersistenceException {
        FileInputStream inFile = null;
        MessageBean bean = null;
        try {
            inFile = new FileInputStream(messageDir + "/message_" + key + ".xml");
            bean = new MessageBean();
            try {
                bean = MessageBean.unmarshal(inFile);
            } finally {
                inFile.close();
            }
        } catch (javax.xml.bind.UnmarshalException e) {
            throw new PersistenceException(e);
        } catch (IOException e) {
            throw new PersistenceException(e);   
        } finally {
            if (inFile != null) {
                try {
                    inFile.close();
                } catch (IOException e) {
                    throw new PersistenceException(e);   
                }
            }
        }
        if (bean == null) {
            throw new PersistenceException("Unable to retrieve data from XML file.");
        }
        return bean;
    }
    
    /**
     * Retrieves last messageBean.
     */
    public MessageBean retrieveLast() throws PersistenceException {
        MessageBean bean = null;
        FileInputStream inFile = null;
        try {
            String idx = this.getLastIndex();
            inFile = new FileInputStream(messageDir + "/message_" + idx + ".xml");
            bean = MessageBean.unmarshal(inFile);
        } catch (javax.xml.bind.UnmarshalException e) {
            throw new PersistenceException(e);
        } catch (IOException e) {
            throw new PersistenceException("Unable to read last message: " + e, e);
        } finally {
            if (inFile != null) {
                try {
                    inFile.close();
                } catch (IOException e) {
                    throw new PersistenceException(e);   
                }
            }
        }
        if (bean == null) {
            throw new PersistenceException("Unable to retrieve data from XML file.");
        }
        return bean;
    }
    
    /**
     * Reads the next index from the indexes file, increments it,
     * saves it and returns it. Helper method.
     */
    private String getNextIndex() throws PersistenceException {
        DataInputStream dataIn = null;
        DataOutputStream dataOut = null;
        try {
            File dataFile = new File(messageDir + "/message.count");
            if (!dataFile.exists()) {
                dataOut = new DataOutputStream(new FileOutputStream(dataFile));
                dataOut.writeInt(1);
                dataOut.close();
                return "1";
            }
            dataIn = new DataInputStream(new FileInputStream(dataFile));
            int lastIndex = dataIn.readInt();
            
            dataIn.close();
            dataOut = new DataOutputStream(new FileOutputStream(dataFile));
            dataOut.writeInt(lastIndex + 1);
            dataOut.close();
            return "" + (lastIndex + 1);
        } catch (IOException e) {
            throw new PersistenceException("Unable to create new message - problems reading index file: " + e, e);
        }
    }
    
    /**
     * Retrieves last index from index file (and not from the files in the directory).
     * See getIndexes for a better approach (refactor this!).
     */
    private String getLastIndex() throws PersistenceException {
        DataInputStream dataIn = null;
        try {
            File dataFile = new File(messageDir + "/message.count");
            dataIn = new DataInputStream(new FileInputStream(dataFile));
            if (!dataFile.exists()) {
                throw new FileNotFoundException("Configuration error: 'message.count' file not found.");
            }
            int lastIndex = dataIn.readInt();
            return "" + lastIndex; // this will fail if last message was deleted!
        } catch (IOException e) {
            throw new PersistenceException("Problems reading index file: " + e, e);
        } finally {
            if (dataIn != null) {
                try {
                    dataIn.close();
                } catch (IOException e) {
                    throw new PersistenceException(e);
                }
            }
        }
    }
    
    /**
     * Gets all the indexes of existing files (does not search the index file
     * but retrieves the indexes from the file names directly. This differs from
     * the solution used in getLastIndex. It is more robust and does not depend
     * on an index file. Messages can now be deleted without corrupting the
     * system.
     */
    private int[] getIndexes() {
        File dir = new File(messageDir);
        File[] files = dir.listFiles(new FilenameFilter() {
            public boolean accept(File dir, String fileName) {
                return (new File(dir, fileName)).isFile() 
                       && fileName.endsWith(".xml")     
                       && fileName.startsWith("message_");
            }
        });
        
        int[] indexes = new int[files.length];
        for (int i = 0; i < files.length; i++) {
            // position of index is after "_" and before ".";
            String fileName = files[i].getName();
            int start = fileName.indexOf("_") + 1;
            int end = fileName.indexOf(".", start);
            indexes[i] = Integer.parseInt(fileName.substring(start, end));
        }
        return indexes;
    }
    
    
}
